/* Executa a renderizacao de objetos 3D. Remove superficies ocultas,
faz projecao perspectiva, translada objeto 3D projetado,
define cores, desenha

ILUMINACAO:

Definicao de iluminacao em um plano. Ocorre GPF se um dos vetores for nulo.
Para funcionar corretamente o poligono deve formar um plano (pelo menos
deve possuir 3 vertices distintos), pertencer a apenas um plano e
OS VERTICES DEVEM ESTAR EM SENTIDO ANTI-HORARIO (se usar modelo flat).
Angulos e graus sao definidos de acordo com NANGS.

Convencoes em CRender::mRetCorZ:
Quanto maior Z menor a iluminacao (cor menor, mais proximo a MINCOR).
se Z >= FIMV entao cor eh MINCOR, se Z <= 0 entao cor
eh MAXCOR.

Usa orientacao do SRU:

          A Z  (para dentro da tela)
          |
          *---------> X
          |
          |
          |
          |
          |
          V

          Y

PROJECAO PERPECTIVA (USA Z):

O centro de visao (superficie da tela) no espaco virtual eh uma area quadrada
de RESXxRESY unidades centrada em (0,0,0).

Se Z positivo:
Quanto maior o Z mais proximo do centro de projecao, mais para a frente,
menor e mais distante o objeto do ponto de visao central (observador).
Se Z negativo:
Quanto menor o Z mais distante do centro de projecao, mais para a tras, fora
da tela se o centro de fuga eh o centro da tela, maior o objeto.
Se Z = 0:
Objeto encostado no centro de visao

   Calculo do angulo de visao em Y:
                                         I              P
                       ------------------+--------------+------ Y
                                         |             /
                                         |Z           /
 Exemplo:                                |           /
 |OI| = 1 (se varia 1 em Z,              |          /
 |IP| = quanto Y varia, incremento       |A'       /
                                         |--------/
 |OP| = sqrt ( |OI|^2 + |IP|^2 )         |       /
                                         *-------
 A' = ? = angulo entre OI e OP           O       C
                                      (observador)
 A' = arco_seno ( |IP| / |OP| )
 A  = angulo de visao em Y = 2 * A'

 	Calculo do angulo de visao em X eh semelhante.

Formula de projecao: f(Z) = 1 / ( Z / distancia_ate_interseccao_IO-CP + 1 ),
onde distancia_ate_interseccao_IO-CP eh o centro de projecao, onde as
projetantes se encontram

O valor obtido em f(Z) eh multiplicado pelas componentes X e Y definido as
respectivas componentes 2D projetadas.

Tabela de projecao:
indices nas tabelas: 0 a MAXZ - MINZ (indice para Z = 0 eh o meio)
Exemplo:
seja MINZ = -2000, MAXZ = 2000, distancia = 300:
indice 2002, Z = 2, projecao = 300 / 2 + 1 = 151
indice 2001, Z = 1, projecao = 300 / 1 + 1 = 301
indice 2000, Z = 0, projecao = 300 / 0 + 1 = 300  (valor assumido nesse erro)
indice 1999, Z = -1, projecao = 300 / -1 + 1 = -299
indice 1998, Z = -2, projecao = 300 / -2 + 1 = -149

REMOCAO DE SUPERFICIES TRASEIRAS:

Para fazer remocao de superficies traseiras:
Se produto escalar entre o vetor normal ao plano do poligono e vetor
do observador ateh ponto de fuga for negativo entao a face visivel do
poligono esta para frente, pode ser desenhada, se for positivo entao
a face estah virada para tras e certamente nao serah desenhada.
Se negativo indica que poligono pode ser desenhado, return TRUE, se nao
nao deve desenhar, return FALSE.
Face do poligono eh o lado do poligono que eh exibivel, apesar do poligono
der ser desenhado pela frente ou por atras apenas eh permitido uma opcao
pela otimizacao.

                  /\
 (solido visto   /  \ A (ponto de fuga)
  de cima)       \   \|
                 /\  /| vetor do observador na discao do ponto de fuga:
                /  \/ |                                         (0,0,1)
  (vetor normal)      * (observador)

 Para obter o vetor normal para "fora" do solido (face frontal dos
 poligonos), como na figura, o poligono deve estah com seus vertices no
 sentido anti-horario.
Para a remocao deve ser levado em conta a projecao usada e a posicao do
poligono projetado. Por isso foram usados os componentes X e Y projetados no
calculo do vetor normal.        */



#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "defs.h"

#include "gr.h"
#include "solido.h"

#include "render.h"



#include <dpmi.h>

#include <conio.h>
#include "teclado.h"



// Calcula vetor normal a um poligono.
// Comentarios entre /**/, falha pre-processador.
#define CALC_VET_NORMAL                                                   \
   /* calcula vetor normal aos 2 vetores obtidos */                       \
   m_nNX = nP1Y * nP2Z - nP1Z * nP2Y,                                     \
   m_nNY = nP1Z * nP2X - nP1X * nP2Z,                                     \
   m_nNZ = nP1X * nP2Y - nP1Y * nP2X;                                     \
   /* (bugfix) se os componentes sao muito grandes reduz a 1/4 o vetor    \
   if ( abs ( m_nNX ) > 15000 || abs ( m_nNX ) > 15000 || abs ( m_nNX ) > 15000 ) { \
		m_nNX = m_nNX >> 2;                                                 \
		m_nNY = m_nNY >> 2;                                                 \
		m_nNZ = m_nNZ >> 2;                                                 \
   }                                                                      \
   /* calcula modulo */                                                   \
   m_fModV = MODULO ( m_nNX, m_nNY, m_nNZ );                              \
   /* (bugfix) verifica se houve erro de calculo: modulo v. normal == 0 */\
   if ( m_fModV == 0 )                                                    \
   	m_fModV = 0.00001;



/* construtor - Calcula tabelas internas, define iluminacao
**********************************************************/
CRender::CRender ()
{
	// calcula tabelas de valores para Z projetados em X e Y
   float
      fAfastamento = 0,
      fDistancia = 300;
   INT
   	nMeio = ( MAXZ - MINZ ) / 2;
	for ( INT16 ncZ = nMeio; ncZ <= ( nMeio + MAXZ ); ncZ ++ ) {

      // valores de 0 a MINZ
      float
      	fVal = - fAfastamento / fDistancia + 1;
      if ( fVal != 0 )
			m_afPrj [ nMeio - ( ncZ - nMeio ) ] = 1 / fVal;
      else
			m_afPrj [ nMeio - ( ncZ - nMeio ) ] = 1;
      // valores de 0 a MAXZ
     	fVal = fAfastamento / fDistancia + 1;
      if ( fVal != 0 )
			m_afPrj [ ncZ ] = 1 / fVal;
      else
			m_afPrj [ ncZ ] = 1;

      fAfastamento += 1;
   }

	// inicia componentes do vetor de iluminacao com 0
	m_nIX = m_nIY = m_nIZ = 0;
	m_fModI = m_fModV = 0.0;

   // nenhum vertice a ser transladado e projetado
	m_panX2D = m_panY2D = NULL;
}



/* destrutor - desaloca memoria
**********************************************************/
CRender::~CRender ()
{
	if ( m_panX2D != NULL )
   	delete m_panX2D;
	if ( m_panY2D != NULL )
   	delete m_panY2D;
}



/* Define objeto que serah renderizado, aloca memoria para
suas componentes transformadas para 2D e calcula tabela de
cores. Deve ser chamado 1 vez antes de inicial renderizacoes.
**********************************************************/
void CRender::mAtrObjeto ( CSolido &oS )
{
	m_poS = &oS;

	// tenta alocar memoria para vertices, se nao conseguir sai
	if ( ( m_panX2D = new INT16 [ oS.mRetMaxVertices () + oS.mRetMaxPoligonos () * 2 ] ) == NULL ) {
		printf ( MSG_SEMMEMORIA );
		exit ( 1 );
	}
	if ( ( m_panY2D = new INT16 [ oS.mRetMaxVertices () + oS.mRetMaxPoligonos () * 2 ] ) == NULL ) {
		printf ( MSG_SEMMEMORIA );
		exit ( 1 );
	}

   // pre-calcula cores dos poligonos conforme posicao da fonte de iluminacao
   /*for ( UINT16 ucP = 0; ucP < oS.m_uNumPoligonos; ucP ++ ) {

      // define quais vertices do poligono nao estao sobrepostos
	   UINT16
         uVNC1 = * ( m_poS->m_pauVNC1 + ucP ),
         uVNC2 = * ( m_poS->m_pauVNC2 + ucP ),
         uVNC3 = * ( m_poS->m_pauVNC3 + ucP );
      // define 2 vetores nao colineares pertencentes ao poligono
	   INT16
		   nP1X = * ( m_poS->m_panX + uVNC1 ) - * ( m_poS->m_panX + uVNC2 ),
		   nP1Y = * ( m_poS->m_panY + uVNC1 ) - * ( m_poS->m_panY + uVNC2 ),
		   nP1Z = * ( m_poS->m_panZ + uVNC1 ) - * ( m_poS->m_panZ + uVNC2 ),
		   nP2X = * ( m_poS->m_panX + uVNC1 ) - * ( m_poS->m_panX + uVNC3 ),
		   nP2Y = * ( m_poS->m_panY + uVNC1 ) - * ( m_poS->m_panY + uVNC3 ),
		   nP2Z = * ( m_poS->m_panZ + uVNC1 ) - * ( m_poS->m_panZ + uVNC3 );
		// define vetor normal ao plano do poligono
      CALC_VET_NORMAL;

      // define cor do poligono
      * ( oS.m_payCor + ucP ) = mRetCorN ( ucP );
   }*/
}



/* Define componentes do vetor de iluminacao
**********************************************************/
void CRender::mAtrIlum ( INT16 nVX, INT16 nVY, INT16 nVZ )
{
	m_nIX = nVX;
	m_nIY = nVY;
	m_nIZ = nVZ;
	m_fModI = MODULO ( m_nIX, m_nIY, m_nIZ );
}



/* Define uma cor de acordo com o angulo entre um vetor do pol e
o vetor de iluminacao. return cor calculada mais cor base (menor cor
utilizavel no poligono). Usa modelo semelhante ao flat. A diferenca
eh que o angulo eh calculado nao entre o vetor de iluminacao e o
vetor normal ao plano e sim entre um vetor pertencente ao plano do
poligono e o vetor de iluminacao. Com isso poupa-se tempo de calculo
do vetor normal. O vetor do plano pertence ao plano e nao eh normal
ao plano. O resultado pratico nao eh o mesmo que usar modelo flat.
**********************************************************/
INT16 CRender::mRetCorP ( UINT16 uPol )
{
	// define vetor pertencente ao poligono usando os vertices do poligono
	INT16
		nPX = * ( m_poS->m_panX + * ( m_poS->m_pauVNC1 + uPol ) ) - * ( m_poS->m_panX + * ( m_poS->m_pauVNC2 + uPol ) ),
		nPY = * ( m_poS->m_panY + * ( m_poS->m_pauVNC1 + uPol ) ) - * ( m_poS->m_panY + * ( m_poS->m_pauVNC2 + uPol ) ),
		nPZ = * ( m_poS->m_panZ + * ( m_poS->m_pauVNC1 + uPol ) ) - * ( m_poS->m_panZ + * ( m_poS->m_pauVNC2 + uPol ) );

	// calcula o angulo entre os 2 vetores: iluminacao e poligono
   // define o produto escalar entre os dois vetores
	float
		fProdEsc = nPX * m_nIX + nPY * m_nIY + nPZ * m_nIZ;
   // define o arco-cosseno do angulo entre vetores
   float
   	fV = (float) fProdEsc / ( MODULO ( nPX, nPY, nPZ ) * m_fModI );
   // verifica erros nos calculos (-1 <= arco-cosseno <= 1)
   if ( fV > 1 )
   	fV = 1;
   else
   	if ( fV < -1 )
      	fV = -1;
   // calcula angulo (entre 0 e NANGS2 - 1)
	INT16
		nAngulo = (INT16) ( acos ( fV ) * NANGS2 / Pi );

	// define cor fazendo regra de tres entre angulo e faixa cores utilizaveis
	return ( * ( m_poS->m_payCor + uPol ) + ( NUMCOR - 1 ) * nAngulo / NANGS2 );
}



/* Define uma cor de acordo com o angulo entre um vetor normal
ao poligono e o vetor de iluminacao. return cor calculada mais
cor base (menor cor utilizavel no poligono). Usa modelo flat.
Como vetor normal eh obtido:
Seja u e v dois vetores pertencentes ao plano do poligono
(obtidos por vertices nao colineares do poligono), n o vetor
normal, i, j ,k os componentes de um vetor unitario (iguais a 1).
         | i  j  k|
         |uX uY uZ| = (Ai, Bj, Ck)    => n = (A, B, C)
         |vX vY vZ|   determinante

Quanto maior o angulo entre o vetor normal e o vetor de iluminacao
mais iluminacao recebe o poligono, quanto menor, menor iluminacao:

                     A (vetor normal)
                     |
                     |        iluminacao zero
                 -------------
                 |           |         (retangulo 3D visto de cima)
                 -------------
                     |        iluminacao maxima
                     |
                     V (vetor normal)

                       A
                       | (vetor de iluminacao)
                       |

Para garantir coerencia na obtencao dos vetores normais todos os
poligonos devem ter seus vertices enumerados conforme sentido
anti-horario nas estruturas de dados (arquivos de dados).
**********************************************************/
INT16 CRender::mRetCorN ( UINT16 uPol )
{
 	// calcula o angulo entre os 2 vetores: iluminacao e normal
   // define o produto escalar entre os dois vetores (iluminacao e normal)
	float
		fProdEsc = m_nNX * m_nIX + m_nNY * m_nIY + m_nNZ * m_nIZ;
   // define o arco-cosseno do angulo entre vetores
   float
   	fV = (float) fProdEsc / ( m_fModV * m_fModI );
   // (bugfix) verifica erros nos calculos (deve ser: -1<=arco-cosseno<=1)
   if ( fV > 1 )
   	fV = 1;
   else
   	if ( fV < -1 )
      	fV = -1;
   // calcula angulo (entre 0 e NANGS2 - 1)
	INT16
		nAngulo = (INT16) ( acos ( fV ) * NANGS2 / Pi );

	// define cor fazendo regra de tres entre angulo e faixa cores utilizaveis
	return ( * ( m_poS->m_payCor + uPol ) + ( NUMCOR - 1 ) * nAngulo / NANGS2 );
}



/* Define uma cor de acordo com o valor Z do ponto medio do poligono.
return cor conforme FIMIZ mais cor base (menor cor utilizavel no poligono).
Quanto mais longe (maior Z) mais proximo o valor a cor base.
**********************************************************/
INT16 CRender::mRetCorZ ( UINT16 uPol )
{
   // define Z central do poligono
   // calcula somas dos componentes de cada vertice
	INT
  	   nSomaZ = * ( m_poS->m_panZ + * ( m_poS->m_pauV1 + uPol ) );
  	nSomaZ += * ( m_poS->m_panZ + * ( m_poS->m_pauV2 + uPol ) );
  	nSomaZ += * ( m_poS->m_panZ + * ( m_poS->m_pauV3 + uPol ) );
  	nSomaZ += * ( m_poS->m_panZ + * ( m_poS->m_pauV4 + uPol ) );

   // verifica se o objeto estah na area visivel
   INT16
   	nZ;
   // se estah muito proximo, desenha com cor mais clara
   if ( ( nZ = ( nSomaZ >> 2 ) ) < 0 )
   	return * ( m_poS->m_payCor + uPol ) + NUMCOR - 1;
   else
      // se estah muito longe, desenha com cor mais escura
   	if ( nZ > FIMIZ )
      	return * ( m_poS->m_payCor + uPol );
      else
         // define cor entre maximo e minimo possivel
			return ( * ( m_poS->m_payCor + uPol ) + (INT16) ( (float) - NUMCOR / FIMIZ * nZ + NUMCOR ) );
}



/* Renderiza pontos (vertices,...) do solido: projeta, translada,
marca na memoria virtual de video e exibe. Altera dados 3D do
solido. nTX e nTY informam posicao na tela para onde objeto
projetado deve ser transladado. bExibir define se deve exibir imagem.
**********************************************************/
void CRender::mRenderPixels ( INT nTX, INT nTY, BYTE yCor )
{
   // transforma solido 3D em 2D e marca
   // varre conjunto de pontos do solido operando ponto por ponto
	for ( UINT16 ucV = 0; ucV < m_poS->m_uNumVertices; ucV ++ ) {

      // faz projecao perspectiva
		* ( m_panX2D + ucV ) = (INT16) ( * ( m_poS->m_panX + ucV ) * * ( m_afPrj + * ( m_poS->m_panZ + ucV ) - MINZ ) );
		* ( m_panY2D + ucV ) = (INT16) ( * ( m_poS->m_panY + ucV ) * * ( m_afPrj + * ( m_poS->m_panZ + ucV ) - MINZ ) );

   	// translada
      INT16
      	nX = * ( m_panX2D + ucV ) + nTX,
         nY = * ( m_panY2D + ucV ) + nTY;

		// verifica se esta nos limites da tela, se sim marca
      if ( nX > MINX && nX < MAXX && nY > MINY && nY < MAXY )
			// altera memoria virtual de video
			m_oGr.mAtrVrPixel ( nX, nY, yCor );
   }

   // exibe imagem no video
  	m_oGr.mAtrVdVrSinc ();
}



/* Renderiza poligonos do solido: projeta, translada,
marca na memoria virtual de video e exibe. Altera dados 3D do
solido. nTX e nTY informam posicao na tela para onde objeto
projetado deve ser transladado. bExibir define se deve exibir imagem.
**********************************************************/
void CRender::mRenderPoligonos ( INT nTX, INT nTY )
{
   // ordena poligonos de acordo com a proximidade com a origem
   m_poS->mOrdena ();

   CTeclado
   	oT;
   if ( oT.mRetPress ( T_F5 ) ) {
	   gotoxy ( 1, 1 );
   	printf ( "Vertice 1 no SRC:\nX: %d\nY: %d\nZ: %d\n", * ( m_poS->m_panX ), * ( m_poS->m_panY ), * ( m_poS->m_panZ ) );
      oT.mRetSoltas ();
   }

   // transforma solido 3D em 2D
   // varre conjunto de pontos do solido operando vertice por vertice
	for ( UINT16 ucV = 0; ucV < m_poS->m_uNumVertices; ucV ++ ) {

		// faz projecao perspectiva
		* ( m_panX2D + ucV ) = (INT16) ( * ( m_poS->m_panX + ucV ) * * ( m_afPrj + * ( m_poS->m_panZ + ucV ) - MINZ ) );
		* ( m_panY2D + ucV ) = (INT16) ( * ( m_poS->m_panY + ucV ) * * ( m_afPrj + * ( m_poS->m_panZ + ucV ) - MINZ ) );

   	// translada
     	* ( m_panX2D + ucV ) += nTX,
      * ( m_panY2D + ucV ) += nTY;
   }

   if ( oT.mRetPress ( T_F4 ) ) {
	   gotoxy ( 1, 1 );
   	printf ( "Vertice 1 no SRC projetado:\nX: %d\nY: %d\nZ: %d\n", * ( m_panX2D ), * ( m_panY2D ), * ( m_poS->m_panZ ) );
      oT.mRetSoltas ();
   }

	// Varre conjunto de poligonos do solido operando poligono por poligono.
   // Comeca dos poligonos mais proximos ateh os mais distantes.
	for ( UINT16 ucP = 0; ucP < m_poS->m_uNumPoligonos; ucP ++ ) {

      // define indice do poligono que serah desenhado
      UINT16
      	uNdxProx = * ( m_poS->m_pauP + ucP );

      // faz recorte parcial
      // verifica se poligono estah todo atras do observador (Z<0)
      // vertices 3D do poligono
   	UINT16
      	uV1 = * ( m_poS->m_pauV1 + uNdxProx ),
      	uV2 = * ( m_poS->m_pauV2 + uNdxProx ),
      	uV3 = * ( m_poS->m_pauV3 + uNdxProx ),
      	uV4 = * ( m_poS->m_pauV4 + uNdxProx );
      // testa se o componente Z dos 4 vertices eh menor que 0, se sim aborta
		if ( * ( m_poS->m_panZ + uV1 ) < -298 || * ( m_poS->m_panZ + uV2 ) < -298 ||
           * ( m_poS->m_panZ + uV3 ) < -298 || * ( m_poS->m_panZ + uV4 ) < -298 )
      	continue;
   	// verifica se o poligono estah completamente fora da tela, se sim sai
      // define vertices 2D projetados e transladados que fazem poligono
      INT16
      	nX2D1 = * ( m_panX2D + uV1 ),
      	nY2D1 = * ( m_panY2D + uV1 ),
      	nX2D2 = * ( m_panX2D + uV2 ),
      	nY2D2 = * ( m_panY2D + uV2 ),
      	nX2D3 = * ( m_panX2D + uV3 ),
      	nY2D3 = * ( m_panY2D + uV3 ),
      	nX2D4 = * ( m_panX2D + uV4 ),
      	nY2D4 = * ( m_panY2D + uV4 );
	   // verifica se todos os componentes x estao a esquerda da tela
      if ( nX2D1 < MINX && nX2D2 < MINX && nX2D3 < MINX && nX2D4 < MINX )
      	continue;               // aborta desenho do poligono
	   // verifica se todos os componentes x estao a direita da tela
      if ( nX2D1 > MAXX && nX2D2 > MAXX && nX2D3 > MAXX && nX2D4 > MAXX )
      	continue;               // aborta desenho do poligono
	   // verifica se todos os componentes y estao acima da tela
      if ( nY2D1 < MINY && nY2D2 < MINY && nY2D3 < MINY && nY2D4 < MINY )
      	continue;               // aborta desenho do poligono
	   // verifica se todos os componentes y estao abaixo da tela
      if ( nY2D1 > MAXY && nY2D2 > MAXY && nY2D3 > MAXY && nY2D4 > MAXY )
      	continue;               // aborta desenho do poligono

      // calcula vetor normal usado na remocao de superficies ocultas
      // define quais vertices nao estao sobrepostos do poligono
	   UINT16
         uVNC1 = * ( m_poS->m_pauVNC1 + uNdxProx ),
         uVNC2 = * ( m_poS->m_pauVNC2 + uNdxProx ),
         uVNC3 = * ( m_poS->m_pauVNC3 + uNdxProx );
      // define 2 vetores nao colineares pertencentes ao poligono
	   INT16
		   nP1X = * ( m_panX2D + uVNC1 ) - * ( m_panX2D + uVNC2 ),
		   nP1Y = * ( m_panY2D + uVNC1 ) - * ( m_panY2D + uVNC2 ),
		   nP1Z = * ( m_poS->m_panZ + uVNC1 ) - * ( m_poS->m_panZ + uVNC2 ),
		   nP2X = * ( m_panX2D + uVNC1 ) - * ( m_panX2D + uVNC3 ),
		   nP2Y = * ( m_panY2D + uVNC1 ) - * ( m_panY2D + uVNC3 ),
		   nP2Z = * ( m_poS->m_panZ + uVNC1 ) - * ( m_poS->m_panZ + uVNC3 );
		// define vetor normal ao plano do poligono
      CALC_VET_NORMAL;

      // faz remocao de superficies traseiras
	 	// Calcula produto escalar entre os 2 vetores: observador e normal ao
		// poligono. Produto escalar = (XN,YN,ZN) * (0,0,1) = Z. Testa valor,
      // se positivo pode ser exibido e negativo certamente nao serah visivel
      if ( m_nNZ > 0 )

		   m_oGr.mAtrVrPol2D ( nX2D1, nY2D1, nX2D2, nY2D2,
			                    nX2D3, nY2D3, nX2D4, nY2D4,
									  // para cores calculadas no momento:
									  mRetCorN ( uNdxProx )
									  // para cores pre-calculadas:
									  /** ( m_poS->m_payCor + uNdxProx )*/ );
   }

   // exibe imagem no video
  	m_oGr.mAtrVdVrSinc ();
}
