/* Transformacoes 3D (translacao, rotacao, escala). Executa transformacoes
sobre solidos ou particoes de solidos.

SISTEMAS DE COORDENADAS:

 Sistema 2D (tela = viewport)
 (0,0) - posicao da origem na tela
   *----->X
   |
  Y|
   |
   V

 Sistema 3D (espaco virtual - universo - SRU):
Como para desenharmos um objeto aparecendo na tela eh necessario translada-lo
ateh uma posicao dentro da area de visao da tela podemos dizer que a posicao
da tela eh fixa e o espaco tem coordenadas relativas a essa origem. Posicao
da tela eh (0,0,0), fica exatamente na origem da tela. A posicao de um objeto
                     depende de onde o objeto 3D for projetado e transladado.
   +-----+-->X       Se um objeto que foi
   | \ /             definido na origem (centro na origem), for projetado e o
  Y|  *Z  *=(0,0,0)  objeto 2D resultante for transladado para
   | / \    (origem) X = 160 e Y = 100 entao a origem do sistema de
   +                 coordenadas do espaco no plano da tela serah (160, 100).
   |  A              Falamos: o objeto foi projetado para o meio da tela
	V	|  				do modo grafico default (320x200).
      |
    exemplo

intervalo para X: limites de INT16 (se 16 bits: -32768 a 32767)
intervalo para Y: limites de INT16 (se 16 bits: -32768 a 32767)
intervalo para Z: MINZ ateh MAXZ (com projecoes calculadas).

ROTACOES:

Cada coordenada pode ser rotacionada nos planos XY, XZ e YZ.

Rotacoes em ZX:       Rotacoes em XY:      Rotacoes no plano YZ:
  (longe)                 /<-\                   /-\
  /--->--\               /    \                  | |
 |        |--(0 graus)   |    |          (perto) A V (longe)
  \---<--/               \    /                  | |
  (perto)                 \->/(Z nao muda)       \-/

Angulos estao medidos de 0 a NANGS - 1.
*/



#include <string.h>
#include <math.h>
#include <stdio.h>

#include "defs.h"

#include "solido.h"

#include "arquivo.h"

#include "universo.h"

#include "sr.h"

#include "transf.h"



/* construtor - Calcula tabelas internas
**********************************************************/
CTransf::CTransf ()
{
	// tabela de senos e cossenos
	for ( INT16 ncAng = 0; ncAng < NANGS; ncAng ++ )	{
      // tabela para ponto fixo
		m_anSeno [ ncAng ] = (INT) ( sin ( ncAng * Pi / NANGS * 2 ) * 65536 );
		m_anCoss [ ncAng ] = (INT) ( cos ( ncAng * Pi / NANGS * 2 ) * 65536 );
      // tabela para ponto flutuante
		m_afSeno [ ncAng ] = sin ( ncAng * Pi / NANGS * 2 );
		m_afCoss [ ncAng ] = cos ( ncAng * Pi / NANGS * 2 );
	}
}



/* Rotaciona pontos definidos no espaco 3D conforme
os 3 angulos (um para cada plano coordenado).
**********************************************************/
void CTransf::mRotaciona ( BYTE yAXY, BYTE yAXZ, BYTE yAYZ,
                           float& fX, float& fY, float& fZ )
{
	float
		fXAux, fYAux, fZAux;

	fXAux = fX    * * ( m_afCoss + yAXY ) + fY    * * ( m_afSeno + yAXY );
	fYAux = fY    * * ( m_afCoss + yAXY ) - fX    * * ( m_afSeno + yAXY );
	fY    = fYAux * * ( m_afCoss + yAYZ ) + fZ    * * ( m_afSeno + yAYZ );
	fZAux = fZ    * * ( m_afCoss + yAYZ ) - fYAux * * ( m_afSeno + yAYZ );
	fX    = fXAux * * ( m_afCoss + yAXZ ) + fZAux * * ( m_afSeno + yAXZ );
	fZ    = fZAux * * ( m_afCoss + yAXZ ) - fXAux * * ( m_afSeno + yAXZ );
}



/* Rotaciona os pontos de um solido conforme os 3 angulos
(um para cada plano coordenado). Os pontos rotacionados sao
colocados diretamente no solido intermediario oSRot.
**********************************************************/
void CTransf::mRotaciona ( BYTE yAXY, BYTE yAXZ, BYTE yAYZ,
                           CSolido &oS, CSolido &oSRot )
{
	INT16
		nXAux, nYAux, nZAux;

	// faz rotacao de cada vertice do poligono
	for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

		nXAux                    = ( * ( oS.m_panX + ucV ) * * ( m_anCoss + yAXY ) >> 16 ) + ( * ( oS.m_panY + ucV ) * * ( m_anSeno + yAXY ) >> 16 );
		nYAux                    = ( * ( oS.m_panY + ucV ) * * ( m_anCoss + yAXY ) >> 16 ) - ( * ( oS.m_panX + ucV ) * * ( m_anSeno + yAXY ) >> 16 );
		* ( oSRot.m_panY + ucV ) = ( nYAux                 * * ( m_anCoss + yAYZ ) >> 16 ) + ( * ( oS.m_panZ + ucV ) * * ( m_anSeno + yAYZ ) >> 16 );
		nZAux                    = ( * ( oS.m_panZ + ucV ) * * ( m_anCoss + yAYZ ) >> 16 ) - ( nYAux                 * * ( m_anSeno + yAYZ ) >> 16 );
		* ( oSRot.m_panX + ucV ) = ( nXAux                 * * ( m_anCoss + yAXZ ) >> 16 ) + ( nZAux                 * * ( m_anSeno + yAXZ ) >> 16 );
		* ( oSRot.m_panZ + ucV ) = ( nZAux                 * * ( m_anCoss + yAXZ ) >> 16 ) - ( nXAux                 * * ( m_anSeno + yAXZ ) >> 16 );
   }

   // define numero de vertices do objeto solido rotacionado
   oSRot.m_uNumVertices = oS.m_uNumVertices;
}



/* Faz translacao de solido 3D.
**********************************************************/
void CTransf::mTranslada ( INT16 nTX, INT16 nTY, INT16 nTZ, CSolido &oS )
{
	// altera cada um dos vertices do solido
	for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

		* ( oS.m_panX + ucV ) += nTX;
		* ( oS.m_panY + ucV ) += nTY;
		* ( oS.m_panZ + ucV ) += nTZ;
	}
}



/* Faz escala de solido
**********************************************************/
void CTransf::mEscala ( float fEX, float fEY, float fEZ, CSolido &oS )
{
	// altera cada um dos vertices do solido
	for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

		* ( oS.m_panX + ucV ) = (INT16) ( (float) * ( oS.m_panX + ucV ) * fEX );
		* ( oS.m_panY + ucV ) = (INT16) ( (float) * ( oS.m_panY + ucV ) * fEY );
		* ( oS.m_panZ + ucV ) = (INT16) ( (float) * ( oS.m_panZ + ucV ) * fEZ );
	}
}



/* Rotaciona os pontos de uma particao de solido conforme os
3 angulos (um para cada plano coordenado). Os pontos originais
sao sobre escritos pelos rotacionados. Default uParte = 0
**********************************************************/
void CTransf::mRotaciona ( BYTE yAXY, BYTE yAXZ, BYTE yAYZ,
									CUniverso& oU, UINT16 uParte )
{
	INT16
		nXAux, nYAux, nZAux;

   // define primeiro e ultimo vertice da faixa de vertices a ser transfomada
   UINT16
   	uIni = * ( oU.m_pauPaIni + uParte ),
   	uFim = * ( oU.m_pauPaFim + uParte );

	// faz rotacao de cada vertice da particao
	for ( UINT16 ucV = uIni; ucV <= uFim; ucV ++ ) {

		nXAux                 = ( * ( oU.m_panX + ucV ) * * ( m_anCoss + yAXY ) >> 16 ) + ( * ( oU.m_panY + ucV ) * * ( m_anSeno + yAXY ) >> 16 );
		nYAux                 = ( * ( oU.m_panY + ucV ) * * ( m_anCoss + yAXY ) >> 16 ) - ( * ( oU.m_panX + ucV ) * * ( m_anSeno + yAXY ) >> 16 );
		* ( oU.m_panY + ucV ) = ( nYAux                 * * ( m_anCoss + yAYZ ) >> 16 ) + ( * ( oU.m_panZ + ucV ) * * ( m_anSeno + yAYZ ) >> 16 );
		nZAux                 = ( * ( oU.m_panZ + ucV ) * * ( m_anCoss + yAYZ ) >> 16 ) - ( nYAux                 * * ( m_anSeno + yAYZ ) >> 16 );
		* ( oU.m_panX + ucV ) = ( nXAux                 * * ( m_anCoss + yAXZ ) >> 16 ) + ( nZAux                 * * ( m_anSeno + yAXZ ) >> 16 );
		* ( oU.m_panZ + ucV ) = ( nZAux                 * * ( m_anCoss + yAXZ ) >> 16 ) - ( nXAux                 * * ( m_anSeno + yAXZ ) >> 16 );
   }
}



/* Faz translacao de uma particao do solido 3D. Default uParte = 0
**********************************************************/
void CTransf::mTranslada ( INT16 nTX, INT16 nTY, INT16 nTZ,
									CUniverso& oU, UINT16 uParte )
{
   // define primeiro e ultimo vertice da faixa de vertices a ser transfomada
   UINT16
   	uIni = * ( oU.m_pauPaIni + uParte ),
   	uFim = * ( oU.m_pauPaFim + uParte );

	// faz translacao de cada vertice da particao
	for ( UINT16 ucV = uIni; ucV <= uFim; ucV ++ ) {

		* ( oU.m_panX + ucV ) += nTX;
		* ( oU.m_panY + ucV ) += nTY;
		* ( oU.m_panZ + ucV ) += nTZ;
	}
}



/* Faz escala de particao.
**********************************************************/
void CTransf::mEscala ( float fEX, float fEY, float fEZ,
								CUniverso& oU, UINT16 uParte )
{
   // define primeiro e ultimo vertice da faixa de vertices a ser transfomada
   UINT16
   	uIni = * ( oU.m_pauPaIni + uParte ),
   	uFim = * ( oU.m_pauPaFim + uParte );

	// altera cada um dos vertices da particao
	for ( UINT16 ucV = uIni; ucV <= uFim; ucV ++ ) {

		* ( oU.m_panX + ucV ) = (INT16) ( (float) * ( oU.m_panX + ucV ) * fEX );
		* ( oU.m_panY + ucV ) = (INT16) ( (float) * ( oU.m_panY + ucV ) * fEY );
		* ( oU.m_panZ + ucV ) = (INT16) ( (float) * ( oU.m_panZ + ucV ) * fEZ );
	}
}



/* Rotaciona os pontos de um solido conforme os 3 angulos
(um para cada plano coordenado). Os pontos originais sao
sobrepostos pelos rotacionados.
**********************************************************/
void CTransf::mRotaciona ( BYTE yAXY, BYTE yAXZ, BYTE yAYZ,
                           CSolido &oS )
{
	INT16
		nXAux, nYAux, nZAux;

	// faz rotacao de cada vertice do poligono
	for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

		nXAux                 = ( * ( oS.m_panX + ucV ) * * ( m_anCoss + yAXY ) >> 16 ) + ( * ( oS.m_panY + ucV ) * * ( m_anSeno + yAXY ) >> 16 );
		nYAux                 = ( * ( oS.m_panY + ucV ) * * ( m_anCoss + yAXY ) >> 16 ) - ( * ( oS.m_panX + ucV ) * * ( m_anSeno + yAXY ) >> 16 );
		* ( oS.m_panY + ucV ) = ( nYAux                 * * ( m_anCoss + yAYZ ) >> 16 ) + ( * ( oS.m_panZ + ucV ) * * ( m_anSeno + yAYZ ) >> 16 );
		nZAux                 = ( * ( oS.m_panZ + ucV ) * * ( m_anCoss + yAYZ ) >> 16 ) - ( nYAux                 * * ( m_anSeno + yAYZ ) >> 16 );
		* ( oS.m_panX + ucV ) = ( nXAux                 * * ( m_anCoss + yAXZ ) >> 16 ) + ( nZAux                 * * ( m_anSeno + yAXZ ) >> 16 );
		* ( oS.m_panZ + ucV ) = ( nZAux                 * * ( m_anCoss + yAXZ ) >> 16 ) - ( nXAux                 * * ( m_anSeno + yAXZ ) >> 16 );
   }
}



/* Translada os pontos de um solido, copia valores para outro solido
**********************************************************/
void CTransf::mTranslada ( INT16 nTX, INT16 nTY, INT16 nTZ, CSolido& oS,
                           CSolido& oSTr )
{
	// altera cada um dos vertices do solido
	for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

		* ( oSTr.m_panX + ucV ) = * ( oS.m_panX + ucV ) + nTX;
		* ( oSTr.m_panY + ucV ) = * ( oS.m_panY + ucV ) + nTY;
		* ( oSTr.m_panZ + ucV ) = * ( oS.m_panZ + ucV ) + nTZ;
	}
}



/* Rotaciona vetores do sistema de referencia conforme seus
 proprios valores (relativos a si mesmos)
**********************************************************/
void CTransf::mRotaciona ( BYTE yAXY, BYTE yAXZ, BYTE yAYZ, CSR& oSR )

{
	float
   	fUX,
		fUY,
		fUZ,
   	fVX,
		fVY,
		fVZ,
   	fWX,
		fWY,
		fWZ;

   fUX = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXk;
   fUY = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYk;
   fUZ = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZk;
   fVX = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fXj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXk;
   fVY = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fYj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYk;
   fVZ = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fZj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZk;
   fWX = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fXj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXk;
   fWY = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fYj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYk;
   fWZ = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fZj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZk;

   oSR.m_fXi = fUX;
   oSR.m_fYi = fUY;
   oSR.m_fZi = fUZ;
   oSR.m_fXj = fVX;
   oSR.m_fYj = fVY;
   oSR.m_fZj = fVZ;
   oSR.m_fXk = fWX;
   oSR.m_fYk = fWY;
   oSR.m_fZk = fWZ;
}



/* Rotaciona vetores do sistema de referencia conforme outro SR.
**********************************************************/
void CTransf::mRotaciona ( BYTE yAXY, BYTE yAXZ, BYTE yAYZ, CSR& oSR, CSR& oSRRot )

{
   oSRRot.m_fXi = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXk;
   oSRRot.m_fYi = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYk;
   oSRRot.m_fZi = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZk;
   oSRRot.m_fXj = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fXj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXk;
   oSRRot.m_fYj = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fYj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYk;
   oSRRot.m_fZj = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fZj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZk;
   oSRRot.m_fXk = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fXj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXk;
   oSRRot.m_fYk = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fYj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYk;
   oSRRot.m_fZk = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fZj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZk;
/*   oSRRot.m_fXi = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXk;
   oSRRot.m_fXj = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYk;
   oSRRot.m_fXk = - ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZi + ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZk;
   oSRRot.m_fYi = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fXj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXk;
   oSRRot.m_fYj = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fYj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYk;
   oSRRot.m_fYk = - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZi + ( * ( m_afCoss + yAYZ )  ) * oSR.m_fZj + - ( * ( m_afSeno + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZk;
   oSRRot.m_fZi = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fXi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fXj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fXk;
   oSRRot.m_fZj = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fYi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fYj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fYk;
   oSRRot.m_fZk = + ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afSeno + yAXZ )  ) * oSR.m_fZi + ( * ( m_afSeno + yAYZ )  ) * oSR.m_fZj +   ( * ( m_afCoss + yAYZ )  ) * ( * ( m_afCoss + yAXZ )  ) * oSR.m_fZk;*/
}



/* Rotaciona sistema de referencia conforme base do universo
**********************************************************/
void CTransf::mRotacionaAbs ( BYTE yAXY, BYTE yAXZ, BYTE yAYZ, CSR& oSR )
{
	float
   		fXAux, fYAux, fZAux;

  	fXAux     = oSR.m_fXi * * ( m_afCoss + yAXY ) + oSR.m_fYi * * ( m_afSeno + yAXY );
 	fYAux     = oSR.m_fYi * * ( m_afCoss + yAXY ) - oSR.m_fXi * * ( m_afSeno + yAXY );
  	oSR.m_fYi = fYAux     * * ( m_afCoss + yAYZ ) + oSR.m_fZi * * ( m_afSeno + yAYZ );
	fZAux     = oSR.m_fZi * * ( m_afCoss + yAYZ ) - fYAux     * * ( m_afSeno + yAYZ );
   oSR.m_fXi = fXAux     * * ( m_afCoss + yAXZ ) + fZAux     * * ( m_afSeno + yAXZ );
   oSR.m_fZi = fZAux     * * ( m_afCoss + yAXZ ) - fXAux     * * ( m_afSeno + yAXZ );

  	fXAux     = oSR.m_fXj * * ( m_afCoss + yAXY ) + oSR.m_fYj * * ( m_afSeno + yAXY );
 	fYAux     = oSR.m_fYj * * ( m_afCoss + yAXY ) - oSR.m_fXj * * ( m_afSeno + yAXY );
  	oSR.m_fYj = fYAux     * * ( m_afCoss + yAYZ ) + oSR.m_fZj * * ( m_afSeno + yAYZ );
	fZAux     = oSR.m_fZj * * ( m_afCoss + yAYZ ) - fYAux     * * ( m_afSeno + yAYZ );
   oSR.m_fXj = fXAux     * * ( m_afCoss + yAXZ ) + fZAux     * * ( m_afSeno + yAXZ );
   oSR.m_fZj = fZAux     * * ( m_afCoss + yAXZ ) - fXAux     * * ( m_afSeno + yAXZ );

  	fXAux     = oSR.m_fXk * * ( m_afCoss + yAXY ) + oSR.m_fYk * * ( m_afSeno + yAXY );
 	fYAux     = oSR.m_fYk * * ( m_afCoss + yAXY ) - oSR.m_fXk * * ( m_afSeno + yAXY );
  	oSR.m_fYk = fYAux     * * ( m_afCoss + yAYZ ) + oSR.m_fZk * * ( m_afSeno + yAYZ );
	fZAux     = oSR.m_fZk * * ( m_afCoss + yAYZ ) - fYAux     * * ( m_afSeno + yAYZ );
   oSR.m_fXk = fXAux     * * ( m_afCoss + yAXZ ) + fZAux     * * ( m_afSeno + yAXZ );
   oSR.m_fZk = fZAux     * * ( m_afCoss + yAXZ ) - fXAux     * * ( m_afSeno + yAXZ );
}



/* Aplica matriz de transformacao ao solido:
solido transformado = solido * T * B^^-1
**********************************************************/
void CTransf::mAtrSistema ( INT16 nTX, INT16 nTY, INT16 nTZ, BYTE yAXY,
                            BYTE yL, BYTE yT, CSolido& oS, CSolido& oST )
{
   // faz transformacao sobre cada vertice do solido
	for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {
   	INT16
      	nDX = * ( oS.m_panX + ucV ) - nTX,
      	nDY = * ( oS.m_panY + ucV ) - nTY,
      	nDZ = * ( oS.m_panZ + ucV ) + nTZ;

   	* ( oST.m_panX + ucV ) = - nDX * * ( m_afCoss + yL ) +
                                 nDZ * * ( m_afSeno + yL );

      * ( oST.m_panY + ucV ) = - nDX * * ( m_afSeno + yT ) *
		                                 * ( m_afSeno + yL ) +
                                 nDY * * ( m_afCoss + yT ) +
      							    - nDZ * * ( m_afSeno + yT ) *
                                       * ( m_afCoss + yL );

      * ( oST.m_panZ + ucV ) =   nDX * * ( m_afCoss + yT ) *
		                                 * ( m_afSeno + yL ) +
                                 nDY * * ( m_afSeno + yT ) +
      							      nDZ * * ( m_afCoss + yT ) *
                                       * ( m_afCoss + yL );
   }

   // define numero de vertices do objeto solido rotacionado
   oST.m_uNumVertices = oS.m_uNumVertices;
}
